
// DllTestDlg.cpp : implementation file
//

#include "stdafx.h"
#include "DllTest.h"
#include "DllTestDlg.h"
#include "afxdialogex.h"

#include "..\\commdll\\commdll.h"

#pragma comment(lib, "..\\Build\\commdll.lib")

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CDllTestDlg dialog



CDllTestDlg::CDllTestDlg(CWnd* pParent /*=NULL*/)
	: CDialogEx(CDllTestDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CDllTestDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_LIST_DEVICES, m_lbDevices);
}

BEGIN_MESSAGE_MAP(CDllTestDlg, CDialogEx)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTON_DISCOVERY, &CDllTestDlg::OnBnClickedButtonDiscovery)
	ON_BN_CLICKED(IDC_BUTTON_CREATE, &CDllTestDlg::OnBnClickedButtonCreate)
	ON_BN_CLICKED(IDC_BUTTON_DESTROY, &CDllTestDlg::OnBnClickedButtonDestroy)
	ON_BN_CLICKED(IDC_BUTTON_CONNECT, &CDllTestDlg::OnBnClickedButtonConnect)
	ON_BN_CLICKED(IDC_BUTTON_DISCONNECT, &CDllTestDlg::OnBnClickedButtonDisconnect)
	ON_BN_CLICKED(IDC_BUTTON_INIT, &CDllTestDlg::OnBnClickedButtonInit)
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_BUTTON_UNINIT, &CDllTestDlg::OnBnClickedButtonUninit)
	ON_BN_CLICKED(IDC_BUTTON_READ, &CDllTestDlg::OnBnClickedButtonRead)
END_MESSAGE_MAP()


// CDllTestDlg message handlers

BOOL CDllTestDlg::OnInitDialog()
{
	CDialogEx::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// TODO: Add extra initialization here
	m_Handle = NULL;
	m_fInit = false;

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CDllTestDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialogEx::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CDllTestDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}



void CDllTestDlg::OnBnClickedButtonDiscovery()
{
	if (!m_fInit)
	{
		AfxMessageBox(_T("Not initialized"));
		return;
	}

	m_lbDevices.ResetContent();

	LPBT_DEVICES pDevices = NULL;
	int iRes = dllDiscovery(&pDevices);
	if (iRes != 0)
	{
		ShowError(_T("Discovering failed"), iRes);
		return;
	}
	
	AfxMessageBox(_T("Discovering completed"));

	for (INT_PTR i = 0; i < pDevices->cCount; i++)
		m_lbDevices.AddString(pDevices->Mac[i]);
	
	dllFreeMem(pDevices);
}


void CDllTestDlg::OnBnClickedButtonCreate()
{
	if (!m_fInit)
	{
		AfxMessageBox(_T("Not initialized"));
		return;
	}

	if (m_Handle)
	{
		AfxMessageBox(_T("Already created"));
		return;
	}

	int i = m_lbDevices.GetCurSel();
	if (i == LB_ERR)
	{
		AfxMessageBox(_T("Select device"));
		return;
	}

	CString sAddress;
	m_lbDevices.GetText(i, sAddress);

	int iRes = dllCreateClient(sAddress.GetBuffer(), &m_Handle);
	if (iRes != 0)
	{
		ShowError(_T("Unable create"), iRes);
		return;
	}

	AfxMessageBox(_T("Created"));
}


void CDllTestDlg::OnBnClickedButtonDestroy()
{
	if (!m_fInit)
	{
		AfxMessageBox(_T("Not initialized"));
		return;
	}

	if (!m_Handle)
	{
		AfxMessageBox(_T("Not created"));
		return;
	}

	int iRes = dllDestroyClient(m_Handle);
	if (iRes != 0)
	{
		ShowError(_T("Unable destroy"), iRes);
		return;
	}

	m_Handle = NULL;

	AfxMessageBox(_T("Destroyed"));
}


void CDllTestDlg::OnBnClickedButtonConnect()
{
	if (!m_fInit)
	{
		AfxMessageBox(_T("Not initialized"));
		return;
	}

	if (!m_Handle)
	{
		AfxMessageBox(_T("Not created"));
		return;
	}

	int iRes = dllConnect(m_Handle);
	if (iRes != 0)
	{
		ShowError(_T("Unable connect"), iRes);
		return;
	}

	AfxMessageBox(_T("Connected"));
}


void CDllTestDlg::OnBnClickedButtonDisconnect()
{
	if (!m_fInit)
	{
		AfxMessageBox(_T("Not initialized"));
		return;
	}

	if (!m_Handle)
	{
		AfxMessageBox(_T("Not created"));
		return;
	}

	int iRes = dllDisconnect(m_Handle);
	if (iRes != 0)
	{
		ShowError(_T("Unable disconnect"), iRes);
		return;
	}

	AfxMessageBox(_T("Disconnected"));
}


void CDllTestDlg::OnBnClickedButtonInit()
{
	if (m_fInit)
	{
		AfxMessageBox(_T("Already initialized"));
		return;
	}

	int iRes = dllInitialize();
	if (iRes != 0)
	{
		ShowError(_T("Unable initialize"), iRes);
		return;
	}

	m_fInit = true;

	AfxMessageBox(_T("Initialized"));
}


void CDllTestDlg::OnDestroy()
{
	CDialogEx::OnDestroy();

	if (m_Handle)
		dllDestroyClient(m_Handle);

	if (m_fInit)
		dllFinalize();
}


void CDllTestDlg::OnBnClickedButtonUninit()
{
	if (!m_fInit)
	{
		AfxMessageBox(_T("Not initialized"));
		return;
	}
	
	if (m_Handle)
	{
		AfxMessageBox(_T("Destroy client"));
		return;
	}

	int iRes = dllFinalize();
	if (iRes != 0)
	{
		ShowError(_T("Unable finalize"), iRes);
		return;
	}

	m_fInit = false;

	AfxMessageBox(_T("Finalized"));
}

void CDllTestDlg::ShowError(CString str, int iRes)
{
	CString s;
	s.Format(_T("%s :: %d"), str, iRes);
	AfxMessageBox(s);
}

void CDllTestDlg::OnBnClickedButtonRead()
{
	if (!m_fInit)
	{
		AfxMessageBox(_T("Not initialized"));
		return;
	}

	if (!m_Handle)
	{
		AfxMessageBox(_T("Not created"));
		return;
	}

	DWORD dwSize = 0;
	LPVOID pData = NULL;

	int iRes = dllGetDataSize(m_Handle, &dwSize);
	if (iRes != 0)
	{
		ShowError(_T("Unable to get available data size"), iRes);
		return;
	}

	if (dwSize == 0)
	{
		AfxMessageBox(_T("No data available"));
		return;
	}

	pData = (LPVOID)LocalAlloc(LPTR, dwSize);
	if (!pData)
	{
		AfxMessageBox(_T("No memory"));
		return;
	}

	iRes = dllRead(m_Handle, pData, &dwSize);
	if (iRes != 0)
	{
		ShowError(_T("Unable read"), iRes);
	}
	else
	{
		CString s = CString((PCHAR)pData);
		AfxMessageBox(s);
	}

	LocalFree(HLOCAL(pData));
}
