#include "stdafx.h"
#include "btcomm.h"

#include "..\\..\\Wireless Communication Library\\dll\\source\\wclUUIDs.h"

//////////////////////////////////////////////////////////////////////////////////////////
//
// CWndThread
//
LRESULT __stdcall CWndThread::_WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	bool fProcessed = false;

	CWndThread* pThread = (CWndThread*)GetWindowLongPtr(hWnd, GWL_USERDATA);
	if (pThread && pThread->m_hWnd == hWnd)
		fProcessed = pThread->WndProc(uMsg, wParam, lParam);
	
	if (fProcessed)
		return 0;

	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

bool CWndThread::WndProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
		case WM_CLOSE:
			DestroyWindow(m_hWnd);
			return true;

		case WM_DESTROY:
			SetWindowLongPtr(m_hWnd, GWL_WNDPROC, (LONG_PTR)DefWindowProc);
			SetWindowLongPtr(m_hWnd, GWL_USERDATA, NULL);

			m_hWnd = NULL;
			PostQuitMessage(0);
			
			return true;

		default:
			return ProcessMessage(uMsg, wParam, lParam);
	}
}

bool CWndThread::ProcessMessage(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	return false;
}

int CWndThread::Initialize()
{
	return WCL_E_SUCCESS;
}

void CWndThread::Finalize()
{
	// Do nothing there
}

UINT __stdcall CWndThread::_ThreadProc(LPVOID lpParam)
{
	if (lpParam)
		((CWndThread*)lpParam)->ThreadProc();

	return 0;
}

void CWndThread::ThreadProc()
{
	m_iThreadInitRes = Initialize();

	if (m_iThreadInitRes == WCL_E_SUCCESS)
	{
		m_hWnd = CreateWindow(_T("Message"), _T("WindowForMessage"), 0, 0, 0, 0, 0, HWND_MESSAGE, NULL, NULL, NULL);
		if (!m_hWnd)
			m_iThreadInitRes = WCL_E_OUT_OF_RESOURCE;
		else
		{
			SetWindowLongPtr(m_hWnd, GWL_USERDATA, (LONG_PTR)this);
			SetWindowLongPtr(m_hWnd, GWL_WNDPROC, (LONG_PTR)_WndProc);
		}
	}

	SetEvent(m_hThreadEvent);

	if (m_iThreadInitRes == WCL_E_SUCCESS)
	{
		MSG msg;
		while (GetMessage(&msg, NULL, 0, 0))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	Finalize();
}

int CWndThread::StartThread()
{
	if (IsRunning())
		return WCL_E_API_LOADED;

	m_hThreadEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
	if (!m_hThreadEvent)
		return WCL_E_OUT_OF_RESOURCE;

	m_hThread = (HANDLE)_beginthreadex(NULL, 0, _ThreadProc, this, 0, NULL);
	if (!m_hThread)
		m_iThreadInitRes = WCL_E_OUT_OF_RESOURCE;
	else
	{
		WaitForSingleObject(m_hThreadEvent, INFINITE);
		if (m_iThreadInitRes != WCL_E_SUCCESS)
		{
			CloseHandle(m_hThread);
			m_hThread = NULL;
		}
	}

	CloseHandle(m_hThreadEvent);
	m_hThreadEvent = NULL;

	return m_iThreadInitRes;
}

int CWndThread::StopThread()
{
	if (!IsRunning())
		return WCL_E_API_NOT_LOADED;

	PostMessage(m_hWnd, WM_CLOSE, 0, 0);
	WaitForSingleObject(m_hThread, INFINITE);

	CloseHandle(m_hThread);
	m_hThread = NULL;

	return WCL_E_SUCCESS;
}

CWndThread::CWndThread()
{
	m_hThread = NULL;
	m_hWnd = NULL;
	m_hThreadEvent = NULL;
	m_iThreadInitRes = WCL_E_SUCCESS;
}

CWndThread::~CWndThread()
{
	StopThread();
}

bool CWndThread::IsRunning()
{
	return (m_hThread != NULL);
}

HWND CWndThread::GetWnd()
{
	return m_hWnd;
}

//////////////////////////////////////////////////////////////////////////////////////////
//
// CBtWndThread
//
CwclBluetoothRadio* CBtWndThread::GetRadio()
{
	CwclBluetoothDiscovery Discovery;
	CwclBluetoothRadios Radios;
	if (Discovery.EnumRadios(Radios) != WCL_E_SUCCESS)
		return NULL;
	
	if (Radios.GetCount() == 0)
		return NULL;

	CwclBluetoothRadio* pRadio = new CwclBluetoothRadio();
	pRadio->Assign(*(Radios.GetItems(0)));
	return pRadio;
}

//////////////////////////////////////////////////////////////////////////////////////////
//
// CBtOperationWndThread
//
int CBtOperationWndThread::ExecuteOperation(UINT opCode, WPARAM wParam, LPARAM lParam)
{
	if (!IsRunning())
		return WCL_E_API_NOT_LOADED;

	m_hOperationCompleteEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
	if (!m_hOperationCompleteEvent)
		return WCL_E_OUT_OF_RESOURCE;

	m_iOperationResult = WCL_E_SUCCESS;

	PostMessage(GetWnd(), opCode, wParam, lParam);
	WaitForSingleObject(m_hOperationCompleteEvent, INFINITE);

	CloseHandle(m_hOperationCompleteEvent);
	m_hOperationCompleteEvent = NULL;

	return m_iOperationResult;
}

void CBtOperationWndThread::SetOperationResult(int iRes)
{
	if (!m_hOperationCompleteEvent)
		return;

	m_iOperationResult = iRes;
	SetEvent(m_hOperationCompleteEvent);
}

CBtOperationWndThread::CBtOperationWndThread()
{
	m_iOperationResult = WCL_E_SUCCESS;
	m_hOperationCompleteEvent = NULL;
}

//////////////////////////////////////////////////////////////////////////////////////////
//
// CBtDiscovery
//
CBtDiscovery::CBtDiscovery()
{
	m_pDevices = NULL;
	m_uDiscoveryMsg = 0;
	m_pDiscovery = NULL;
}

int CBtDiscovery::Discovery(LPBT_DEVICES* ppDevices)
{
	*ppDevices = NULL;

	if (IsRunning())
		return WCL_E_API_LOADED;

	int iRes = StartThread();
	if (iRes != WCL_E_SUCCESS)
		return iRes;

	iRes = ExecuteOperation(m_uDiscoveryMsg, 0, 0);
	if (iRes == WCL_E_SUCCESS)
	{
		// Copy devices
		DWORD dwSize = sizeof(BT_DEVICES);
		if (m_pDevices->GetCount() > 0)
			dwSize = dwSize + sizeof(BT_MAC) * (m_pDevices->GetCount() - 1);
		*ppDevices = (LPBT_DEVICES)LocalAlloc(LPTR, dwSize);
		if (*ppDevices == NULL)
			iRes = WCL_E_OUT_OF_MEMORY;
		else
		{
			(*ppDevices)->cCount = m_pDevices->GetCount();
			if (m_pDevices->GetCount() > 0)
				for (INT_PTR i = 0; i < m_pDevices->GetCount(); i++)
					_tcscpy_s((*ppDevices)->Mac[i], m_pDevices->GetItems(i)->GetAddress());
		}

		delete m_pDevices;
		m_pDevices = NULL;
	}
	
	StopThread();

	return iRes;
}

int CBtDiscovery::StartDiscovery()
{
	CwclBluetoothRadio* pRadio = GetRadio();
	if (!pRadio)
		return WCL_E_BLUETOOTH_API_NOT_AVAILABLE;

	int iRes = m_pDiscovery->Discovery(*pRadio);

	delete pRadio;

	return iRes;
}

bool CBtDiscovery::ProcessMessage(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if (uMsg == m_uDiscoveryMsg)
	{
		int iRes = StartDiscovery();
		if (iRes != WCL_E_SUCCESS)
			SetOperationResult(iRes);

		return true;
	}

	return CBtOperationWndThread::ProcessMessage(uMsg, wParam, lParam);
}

void CBtDiscovery::OnDiscoveryComplete(void* pSender, CwclBluetoothRadio* pRadio, CwclBluetoothDevices* pDevices)
{
	if (!pDevices)
		SetOperationResult(WCL_E_INTERNAL);
	else
	{
		m_pDevices = new CwclBluetoothDevices();
		m_pDevices->Assign(*pDevices);

		SetOperationResult(WCL_E_SUCCESS);
	}
}

int CBtDiscovery::Initialize()
{
	m_uDiscoveryMsg = RegisterWindowMessage(_T("BT_DISCOVERY_MSG"));
	if (!m_uDiscoveryMsg)
		return WCL_E_OUT_OF_RESOURCE;

	m_pDiscovery = new CwclBluetoothDiscovery();
	__hook(&CwclBluetoothDiscovery::OnDiscoveryComplete, m_pDiscovery, &CBtDiscovery::OnDiscoveryComplete);
	
	return WCL_E_SUCCESS;
}

void CBtDiscovery::Finalize()
{
	m_uDiscoveryMsg = 0;

	if (m_pDiscovery)
	{
		if (m_pDiscovery->GetActive())
			m_pDiscovery->Terminate();

		__unhook(&CwclBluetoothDiscovery::OnDiscoveryComplete, m_pDiscovery, &CBtDiscovery::OnDiscoveryComplete);

		delete m_pDiscovery;
		m_pDiscovery = NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////////////////
//
// CBtClient
//
int CBtClient::Initialize()
{
	m_uConnectMsg = RegisterWindowMessage(_T("BT_CONNECT_MSG"));
	if (!m_uConnectMsg)
		return WCL_E_OUT_OF_RESOURCE;
	
	m_uDisconnectMsg = RegisterWindowMessage(_T("BT_DISCONNECT_MSG"));
	if (!m_uDisconnectMsg)
		return WCL_E_OUT_OF_RESOURCE;

	m_uWriteMsg = RegisterWindowMessage(_T("BT_WRITE_MSG"));
	if (!m_uWriteMsg)
		return WCL_E_OUT_OF_RESOURCE;
	
	m_uReadMsg = RegisterWindowMessage(_T("BT_READ_MSG"));
	if (!m_uReadMsg)
		return WCL_E_OUT_OF_RESOURCE;

	m_uGetSizeMsg = RegisterWindowMessage(_T("BT_SIZE_MSG"));
	if (!m_uGetSizeMsg)
		return WCL_E_OUT_OF_RESOURCE;

	m_pClient = new CwclSyncClient();
	__hook(&CwclSyncClient::OnConnect, m_pClient, &CBtClient::OnConnect);
	__hook(&CwclSyncClient::OnDisconnect, m_pClient, &CBtClient::OnDisconnect);

	return WCL_E_SUCCESS;
}

void CBtClient::Finalize()
{
	m_uConnectMsg = 0;
	m_uDisconnectMsg = 0;
	m_uWriteMsg = 0;
	m_uReadMsg = 0;
	m_uGetSizeMsg = 0;

	if (m_pClient)
	{
		if (m_pClient->GetState() != csDisconnected)
			m_pClient->Disconnect();

		__unhook(&CwclSyncClient::OnConnect, m_pClient, &CBtClient::OnConnect);
		__unhook(&CwclSyncClient::OnDisconnect, m_pClient, &CBtClient::OnDisconnect);

		delete m_pClient;
		m_pClient = NULL;
	}
}

void CBtClient::OnConnect(void* pSender, int iError)
{
	SetOperationResult(iError);
}

void CBtClient::OnDisconnect(void* pSender)
{
	// To do nothing.
}

bool CBtClient::ProcessMessage(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if (uMsg == m_uConnectMsg)
	{
		CwclBluetoothRadio* pRadio = GetRadio();
		if (!pRadio)
		{
			SetOperationResult(WCL_E_BLUETOOTH_API_NOT_AVAILABLE);
			return true;
		}

		m_pClient->GetBluetoothParams()->SetAddress(m_sAddress);
		m_pClient->GetBluetoothParams()->SetAuthentication(false);
		m_pClient->GetBluetoothParams()->SetEncryption(false);
		m_pClient->GetBluetoothParams()->SetService(SerialPortServiceClass_UUID);
		m_pClient->GetBluetoothParams()->SetRadio(pRadio);

		int iRes = m_pClient->Connect();
		if (iRes != WCL_E_SUCCESS)
			SetOperationResult(iRes);

		delete pRadio;

		return true;
	}

	if (uMsg == m_uDisconnectMsg)
	{
		m_pClient->Disconnect();
		SetOperationResult(WCL_E_SUCCESS);
		return true;
	}

	if (uMsg == m_uWriteMsg)
	{
		SetOperationResult(m_pClient->Write(m_pData, m_dwSize));
		return true;
	}

	if (uMsg == m_uReadMsg)
	{
		SetOperationResult(m_pClient->Read(m_pData, m_dwSize, INFINITE));
		return true;
	}

	if (uMsg == m_uGetSizeMsg)
	{
		m_dwSize = m_pClient->GetDataSize();
		SetOperationResult(WCL_E_SUCCESS);
		return true;
	}

	return CBtOperationWndThread::ProcessMessage(uMsg, wParam, lParam);
}

CBtClient::CBtClient(CAtlString sAddress)
{
	m_sAddress = sAddress;
	
	m_pClient = NULL;
	
	m_uConnectMsg = 0;
	m_uDisconnectMsg = 0;
	m_uWriteMsg = 0;
	m_uReadMsg = 0;
	m_uGetSizeMsg = 0;
	
	m_pData = NULL;
	m_dwSize = 0;
}

CBtClient::~CBtClient()
{
	Disconnect();
}

void CBtClient::SetTemp(LPVOID pData, DWORD dwSize)
{
	m_pData = pData;
	m_dwSize = dwSize;
}

void CBtClient::ClearTemp()
{
	m_pData = NULL;
	m_dwSize = 0;
}

int CBtClient::Connect()
{
	if (m_pClient != NULL)
		return WCL_E_CONNECTION_ACTIVE;

	int iRes = StartThread();
	if (iRes != WCL_E_SUCCESS)
		return iRes;

	iRes = ExecuteOperation(m_uConnectMsg, 0, 0);
	if (iRes != WCL_E_SUCCESS)
		StopThread();

	return iRes;
}

int CBtClient::Disconnect()
{
	if (!IsConnected())
		return WCL_E_NOT_CONNECTED;

	int iRes = ExecuteOperation(m_uDisconnectMsg, 0, 0);
	if (iRes == WCL_E_SUCCESS)
		StopThread();

	return iRes;
}

int CBtClient::Write(LPVOID pData, DWORD dwSize)
{
	if (!IsConnected())
		return WCL_E_NOT_CONNECTED;

	if (pData == NULL || dwSize == 0)
		return WCL_E_INVALID_ARGUMENT;

	SetTemp(pData, dwSize);
	int iRes = ExecuteOperation(m_uWriteMsg, 0, 0);
	ClearTemp();

	return iRes;
}

int CBtClient::Read(LPVOID pData, LPDWORD pdwSize)
{
	if (!IsConnected())
		return WCL_E_NOT_CONNECTED;

	if (pData == NULL || pdwSize == NULL || (*pdwSize) == 0)
		return WCL_E_INVALID_ARGUMENT;

	SetTemp(pData, (*pdwSize));
	int iRes = ExecuteOperation(m_uReadMsg, 0, 0);
	(*pdwSize) = m_dwSize;
	ClearTemp();

	return iRes;
}

int CBtClient::GetDataSize(LPDWORD pdwSize)
{
	if (!IsConnected())
		return WCL_E_NOT_CONNECTED;

	if (!pdwSize)
		return WCL_E_INVALID_ARGUMENT;

	SetTemp(NULL, 0);
	(*pdwSize) = 0;
	int iRes = ExecuteOperation(m_uGetSizeMsg, 0, 0);
	if (iRes == WCL_E_SUCCESS)
		(*pdwSize) = m_dwSize;
	ClearTemp();

	return iRes;
}

bool CBtClient::IsConnected()
{
	return (m_pClient != NULL && m_pClient->GetState() == csConnected);
}

CAtlString CBtClient::GetAddress()
{
	return m_sAddress;
}