#pragma once

#include "commdll.h"

#include "..\\..\\Wireless Communication Library\\dll\\source\\wclAPI.h"
#include "..\\..\\Wireless Communication Library\\dll\\source\\wclErrors.h"
#include "..\\..\\Wireless Communication Library\\dll\\source\\wclBluetoothDiscovery.h"
#include "..\\..\\Wireless Communication Library\\dll\\source\\wclConnections.h"

//#pragma comment(lib, "..\\..\\Wireless Communication Library\\dll\\build\\x32\\2013\\static\\wcl.lib")

using namespace wcl;

class CWndThread
{
private:
	HANDLE	m_hThread;
	HWND	m_hWnd;
	HANDLE	m_hThreadEvent;
	int		m_iThreadInitRes;

	static LRESULT __stdcall _WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
	bool WndProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

	static UINT __stdcall _ThreadProc(LPVOID lpParam);
	void ThreadProc();

protected:
	HWND GetWnd();

	int StartThread();
	int StopThread();

	virtual int Initialize();
	virtual void Finalize();

	virtual bool ProcessMessage(UINT uMsg, WPARAM wParam, LPARAM lParam);

public:
	CWndThread();
	virtual ~CWndThread();

	bool IsRunning();
};

class CBtWndThread : public CWndThread
{
protected:
	CwclBluetoothRadio* GetRadio();
};

class CBtOperationWndThread : public CBtWndThread
{
private:
	int		m_iOperationResult;
	HANDLE	m_hOperationCompleteEvent;

protected:
	int ExecuteOperation(UINT opCode, WPARAM wParam, LPARAM lParam);
	void SetOperationResult(int iRes);

public:
	CBtOperationWndThread();
};

class CBtDiscovery : public CBtOperationWndThread
{
private:
	CwclBluetoothDevices*	m_pDevices;
	CwclBluetoothDiscovery*	m_pDiscovery;
	
	UINT					m_uDiscoveryMsg;

	int StartDiscovery();

	void OnDiscoveryComplete(void* pSender, CwclBluetoothRadio* pRadio, CwclBluetoothDevices* pDevices);

protected:
	virtual int Initialize();
	virtual void Finalize();

	virtual bool ProcessMessage(UINT uMsg, WPARAM wParam, LPARAM lParam);

public:
	CBtDiscovery();

	int Discovery(LPBT_DEVICES* ppDevices);
};

class CBtClient : public CBtOperationWndThread
{
private:
	CAtlString		m_sAddress;
	CwclSyncClient*	m_pClient;
	
	UINT			m_uConnectMsg;
	UINT			m_uDisconnectMsg;
	UINT			m_uWriteMsg;
	UINT			m_uReadMsg;
	UINT			m_uGetSizeMsg;

	LPVOID			m_pData;
	DWORD			m_dwSize;

	void OnConnect(void* pSender, int iError);
	void OnDisconnect(void* pSender);

	void SetTemp(LPVOID pData, DWORD dwSize);
	void ClearTemp();

protected:
	virtual int Initialize();
	virtual void Finalize();

	virtual bool ProcessMessage(UINT uMsg, WPARAM wParam, LPARAM lParam);

public:
	CBtClient(CAtlString sAddress);
	virtual ~CBtClient();

	int Connect();
	int Disconnect();
	int Write(LPVOID pData, DWORD dwSize);
	int Read(LPVOID pData, LPDWORD pdwSize);
	int GetDataSize(LPDWORD pdwSize);

	bool IsConnected();

	CAtlString GetAddress();
};